#!/usr/bin/env python2
# -*- coding: utf-8 -*-
##################################################
# GNU Radio Python Flow Graph
# Title: GreenCube SSB Transceiver
# Author: Lorenzo Frezza, Diego Amadio
# GNU Radio version: 3.7.13.5
##################################################

if __name__ == '__main__':
    import ctypes
    import sys
    if sys.platform.startswith('linux'):
        try:
            x11 = ctypes.cdll.LoadLibrary('libX11.so')
            x11.XInitThreads()
        except:
            print "Warning: failed to XInitThreads()"

from PyQt4 import Qt
from gnuradio import analog
from gnuradio import audio
from gnuradio import blocks
from gnuradio import digital
from gnuradio import eng_notation
from gnuradio import filter
from gnuradio import gr
from gnuradio import qtgui
from gnuradio.eng_option import eng_option
from gnuradio.filter import firdes
from gnuradio.qtgui import Range, RangeWidget
from optparse import OptionParser
import ConfigParser
import math
import os.path
import sip
import sys
from gnuradio import qtgui


class top_block(gr.top_block, Qt.QWidget):

    def __init__(self):
        gr.top_block.__init__(self, "GreenCube SSB Transceiver")
        Qt.QWidget.__init__(self)
        self.setWindowTitle("GreenCube SSB Transceiver")
        qtgui.util.check_set_qss()
        try:
            self.setWindowIcon(Qt.QIcon.fromTheme('gnuradio-grc'))
        except:
            pass
        self.top_scroll_layout = Qt.QVBoxLayout()
        self.setLayout(self.top_scroll_layout)
        self.top_scroll = Qt.QScrollArea()
        self.top_scroll.setFrameStyle(Qt.QFrame.NoFrame)
        self.top_scroll_layout.addWidget(self.top_scroll)
        self.top_scroll.setWidgetResizable(True)
        self.top_widget = Qt.QWidget()
        self.top_scroll.setWidget(self.top_widget)
        self.top_layout = Qt.QVBoxLayout(self.top_widget)
        self.top_grid_layout = Qt.QGridLayout()
        self.top_layout.addLayout(self.top_grid_layout)

        self.settings = Qt.QSettings("GNU Radio", "top_block")
        self.restoreGeometry(self.settings.value("geometry").toByteArray())


        ##################################################
        # Variables
        ##################################################
        self.configurationFilePath = configurationFilePath = os.path.expanduser("~") + "\GreenCube\config.ini"
        self.rx_baud = rx_baud = 1200
        self._rx_audio_sample_rate_config = ConfigParser.ConfigParser()
        self._rx_audio_sample_rate_config.read(configurationFilePath)
        try: rx_audio_sample_rate = self._rx_audio_sample_rate_config.getfloat('gnuradio', 'AudioCard_TX_SampleRate')
        except: rx_audio_sample_rate = 48000
        self.rx_audio_sample_rate = rx_audio_sample_rate
        self.tx_baud = tx_baud = 1200
        self._tx_audio_sample_rate_config = ConfigParser.ConfigParser()
        self._tx_audio_sample_rate_config.read(configurationFilePath)
        try: tx_audio_sample_rate = self._tx_audio_sample_rate_config.getfloat('gnuradio', 'AudioCard_TX_SampleRate')
        except: tx_audio_sample_rate = 48000
        self.tx_audio_sample_rate = tx_audio_sample_rate
        self.sps = sps = rx_audio_sample_rate/rx_baud
        self.nfilts = nfilts = 32
        self.variable_qtgui_range_0 = variable_qtgui_range_0 = 1
        self.tx_sps = tx_sps = tx_audio_sample_rate/tx_baud
        self._ssb_tx_offset_config = ConfigParser.ConfigParser()
        self._ssb_tx_offset_config.read(configurationFilePath)
        try: ssb_tx_offset = self._ssb_tx_offset_config.getfloat('gnuradio', 'SSB_TX_Offset')
        except: ssb_tx_offset = 1200
        self.ssb_tx_offset = ssb_tx_offset
        self._ssb_rx_offset_config = ConfigParser.ConfigParser()
        self._ssb_rx_offset_config.read(configurationFilePath)
        try: ssb_rx_offset = self._ssb_rx_offset_config.getfloat('gnuradio', 'SSB_RX_Offset')
        except: ssb_rx_offset = 1600
        self.ssb_rx_offset = ssb_rx_offset
        self._ssb_mode_config = ConfigParser.ConfigParser()
        self._ssb_mode_config.read(configurationFilePath)
        try: ssb_mode = self._ssb_mode_config.get('gnuradio', 'SSB_Mode')
        except: ssb_mode = 'USB'
        self.ssb_mode = ssb_mode
        self.rx_sps = rx_sps = rx_audio_sample_rate/rx_baud
        self.rrc_filter = rrc_filter = firdes.root_raised_cosine(nfilts, nfilts, 1.0/float(sps), 0.35, 11*nfilts)
        self.lpf_dec = lpf_dec = 1
        self._gr_transmit_port_config = ConfigParser.ConfigParser()
        self._gr_transmit_port_config.read(configurationFilePath)
        try: gr_transmit_port = self._gr_transmit_port_config.getint('modem', 'GR_TransmitPort')
        except: gr_transmit_port = 55011
        self.gr_transmit_port = gr_transmit_port
        self._gr_receive_port_config = ConfigParser.ConfigParser()
        self._gr_receive_port_config.read(configurationFilePath)
        try: gr_receive_port = self._gr_receive_port_config.getint('modem', 'GR_ReceivePort')
        except: gr_receive_port = 55012
        self.gr_receive_port = gr_receive_port
        self.bw_fact = bw_fact = .6
        self._audiocard_tx_devicename_config = ConfigParser.ConfigParser()
        self._audiocard_tx_devicename_config.read(configurationFilePath)
        try: audiocard_tx_devicename = self._audiocard_tx_devicename_config.get('gnuradio', 'AudioCard_TX_DeviceName')
        except: audiocard_tx_devicename = ''
        self.audiocard_tx_devicename = audiocard_tx_devicename
        self._audiocard_rx_devicename_config = ConfigParser.ConfigParser()
        self._audiocard_rx_devicename_config.read(configurationFilePath)
        try: audiocard_rx_devicename = self._audiocard_rx_devicename_config.get('gnuradio', 'AudioCard_RX_DeviceName')
        except: audiocard_rx_devicename = 'Line 1 (Virtual Audio Cable)'
        self.audiocard_rx_devicename = audiocard_rx_devicename

        ##################################################
        # Blocks
        ##################################################
        self.tab = Qt.QTabWidget()
        self.tab_widget_0 = Qt.QWidget()
        self.tab_layout_0 = Qt.QBoxLayout(Qt.QBoxLayout.TopToBottom, self.tab_widget_0)
        self.tab_grid_layout_0 = Qt.QGridLayout()
        self.tab_layout_0.addLayout(self.tab_grid_layout_0)
        self.tab.addTab(self.tab_widget_0, 'Incoming transmission')
        self.tab_widget_1 = Qt.QWidget()
        self.tab_layout_1 = Qt.QBoxLayout(Qt.QBoxLayout.TopToBottom, self.tab_widget_1)
        self.tab_grid_layout_1 = Qt.QGridLayout()
        self.tab_layout_1.addLayout(self.tab_grid_layout_1)
        self.tab.addTab(self.tab_widget_1, 'Ougoing transmission')
        self.top_grid_layout.addWidget(self.tab, 0, 0, 3, 3)
        for r in range(0, 3):
            self.top_grid_layout.setRowStretch(r, 1)
        for c in range(0, 3):
            self.top_grid_layout.setColumnStretch(c, 1)
        self._variable_qtgui_range_0_range = Range(0, 100, 1, 1, 200)
        self._variable_qtgui_range_0_win = RangeWidget(self._variable_qtgui_range_0_range, self.set_variable_qtgui_range_0, "variable_qtgui_range_0", "counter_slider", float)
        self.top_grid_layout.addWidget(self._variable_qtgui_range_0_win)
        self.qtgui_waterfall_sink_x_0_0_0 = qtgui.waterfall_sink_c(
        	1024, #size
        	firdes.WIN_BLACKMAN_hARRIS, #wintype
        	0, #fc
        	rx_audio_sample_rate/4, #bw
        	"", #name
                1 #number of inputs
        )
        self.qtgui_waterfall_sink_x_0_0_0.set_update_time(0.10)
        self.qtgui_waterfall_sink_x_0_0_0.enable_grid(False)
        self.qtgui_waterfall_sink_x_0_0_0.enable_axis_labels(True)

        if not False:
          self.qtgui_waterfall_sink_x_0_0_0.disable_legend()

        if "complex" == "float" or "complex" == "msg_float":
          self.qtgui_waterfall_sink_x_0_0_0.set_plot_pos_half(not True)

        labels = ['', '', '', '', '',
                  '', '', '', '', '']
        colors = [0, 0, 0, 0, 0,
                  0, 0, 0, 0, 0]
        alphas = [1.0, 1.0, 1.0, 1.0, 1.0,
                  1.0, 1.0, 1.0, 1.0, 1.0]
        for i in xrange(1):
            if len(labels[i]) == 0:
                self.qtgui_waterfall_sink_x_0_0_0.set_line_label(i, "Data {0}".format(i))
            else:
                self.qtgui_waterfall_sink_x_0_0_0.set_line_label(i, labels[i])
            self.qtgui_waterfall_sink_x_0_0_0.set_color_map(i, colors[i])
            self.qtgui_waterfall_sink_x_0_0_0.set_line_alpha(i, alphas[i])

        self.qtgui_waterfall_sink_x_0_0_0.set_intensity_range(-140, 10)

        self._qtgui_waterfall_sink_x_0_0_0_win = sip.wrapinstance(self.qtgui_waterfall_sink_x_0_0_0.pyqwidget(), Qt.QWidget)
        self.tab_grid_layout_0.addWidget(self._qtgui_waterfall_sink_x_0_0_0_win, 1, 0, 1, 1)
        for r in range(1, 2):
            self.tab_grid_layout_0.setRowStretch(r, 1)
        for c in range(0, 1):
            self.tab_grid_layout_0.setColumnStretch(c, 1)
        self.qtgui_waterfall_sink_x_0_0 = qtgui.waterfall_sink_f(
        	4096, #size
        	firdes.WIN_BLACKMAN_hARRIS, #wintype
        	0, #fc
        	tx_audio_sample_rate, #bw
        	"", #name
                1 #number of inputs
        )
        self.qtgui_waterfall_sink_x_0_0.set_update_time(0.10)
        self.qtgui_waterfall_sink_x_0_0.enable_grid(False)
        self.qtgui_waterfall_sink_x_0_0.enable_axis_labels(True)

        if not True:
          self.qtgui_waterfall_sink_x_0_0.disable_legend()

        if "float" == "float" or "float" == "msg_float":
          self.qtgui_waterfall_sink_x_0_0.set_plot_pos_half(not True)

        labels = ['', '', '', '', '',
                  '', '', '', '', '']
        colors = [0, 0, 0, 0, 0,
                  0, 0, 0, 0, 0]
        alphas = [1.0, 1.0, 1.0, 1.0, 1.0,
                  1.0, 1.0, 1.0, 1.0, 1.0]
        for i in xrange(1):
            if len(labels[i]) == 0:
                self.qtgui_waterfall_sink_x_0_0.set_line_label(i, "Data {0}".format(i))
            else:
                self.qtgui_waterfall_sink_x_0_0.set_line_label(i, labels[i])
            self.qtgui_waterfall_sink_x_0_0.set_color_map(i, colors[i])
            self.qtgui_waterfall_sink_x_0_0.set_line_alpha(i, alphas[i])

        self.qtgui_waterfall_sink_x_0_0.set_intensity_range(-140, 10)

        self._qtgui_waterfall_sink_x_0_0_win = sip.wrapinstance(self.qtgui_waterfall_sink_x_0_0.pyqwidget(), Qt.QWidget)
        self.tab_grid_layout_1.addWidget(self._qtgui_waterfall_sink_x_0_0_win, 1, 0, 1, 1)
        for r in range(1, 2):
            self.tab_grid_layout_1.setRowStretch(r, 1)
        for c in range(0, 1):
            self.tab_grid_layout_1.setColumnStretch(c, 1)
        self.qtgui_time_sink_x_0_0 = qtgui.time_sink_f(
        	10240, #size
        	rx_audio_sample_rate, #samp_rate
        	"", #name
        	1 #number of inputs
        )
        self.qtgui_time_sink_x_0_0.set_update_time(0.10)
        self.qtgui_time_sink_x_0_0.set_y_axis(-3, 3)

        self.qtgui_time_sink_x_0_0.set_y_label('Amplitude', "")

        self.qtgui_time_sink_x_0_0.enable_tags(-1, True)
        self.qtgui_time_sink_x_0_0.set_trigger_mode(qtgui.TRIG_MODE_FREE, qtgui.TRIG_SLOPE_POS, 0.0, 0, 0, "")
        self.qtgui_time_sink_x_0_0.enable_autoscale(False)
        self.qtgui_time_sink_x_0_0.enable_grid(True)
        self.qtgui_time_sink_x_0_0.enable_axis_labels(False)
        self.qtgui_time_sink_x_0_0.enable_control_panel(False)
        self.qtgui_time_sink_x_0_0.enable_stem_plot(False)

        if not False:
          self.qtgui_time_sink_x_0_0.disable_legend()

        labels = ['', '', '', '', '',
                  '', '', '', '', '']
        widths = [1, 1, 1, 1, 1,
                  1, 1, 1, 1, 1]
        colors = ["blue", "red", "green", "black", "cyan",
                  "magenta", "yellow", "dark red", "dark green", "blue"]
        styles = [1, 1, 1, 1, 1,
                  1, 1, 1, 1, 1]
        markers = [-1, -1, -1, -1, -1,
                   -1, -1, -1, -1, -1]
        alphas = [1.0, 1.0, 1.0, 1.0, 1.0,
                  1.0, 1.0, 1.0, 1.0, 1.0]

        for i in xrange(1):
            if len(labels[i]) == 0:
                self.qtgui_time_sink_x_0_0.set_line_label(i, "Data {0}".format(i))
            else:
                self.qtgui_time_sink_x_0_0.set_line_label(i, labels[i])
            self.qtgui_time_sink_x_0_0.set_line_width(i, widths[i])
            self.qtgui_time_sink_x_0_0.set_line_color(i, colors[i])
            self.qtgui_time_sink_x_0_0.set_line_style(i, styles[i])
            self.qtgui_time_sink_x_0_0.set_line_marker(i, markers[i])
            self.qtgui_time_sink_x_0_0.set_line_alpha(i, alphas[i])

        self._qtgui_time_sink_x_0_0_win = sip.wrapinstance(self.qtgui_time_sink_x_0_0.pyqwidget(), Qt.QWidget)
        self.tab_grid_layout_0.addWidget(self._qtgui_time_sink_x_0_0_win, 0, 0, 1, 1)
        for r in range(0, 1):
            self.tab_grid_layout_0.setRowStretch(r, 1)
        for c in range(0, 1):
            self.tab_grid_layout_0.setColumnStretch(c, 1)
        self.qtgui_time_sink_x_0 = qtgui.time_sink_f(
        	10240, #size
        	tx_audio_sample_rate, #samp_rate
        	"", #name
        	1 #number of inputs
        )
        self.qtgui_time_sink_x_0.set_update_time(0.10)
        self.qtgui_time_sink_x_0.set_y_axis(-2, 2)

        self.qtgui_time_sink_x_0.set_y_label('Amplitude', "")

        self.qtgui_time_sink_x_0.enable_tags(-1, True)
        self.qtgui_time_sink_x_0.set_trigger_mode(qtgui.TRIG_MODE_FREE, qtgui.TRIG_SLOPE_POS, 0.0, 0, 0, "")
        self.qtgui_time_sink_x_0.enable_autoscale(False)
        self.qtgui_time_sink_x_0.enable_grid(True)
        self.qtgui_time_sink_x_0.enable_axis_labels(False)
        self.qtgui_time_sink_x_0.enable_control_panel(False)
        self.qtgui_time_sink_x_0.enable_stem_plot(False)

        if not False:
          self.qtgui_time_sink_x_0.disable_legend()

        labels = ['', '', '', '', '',
                  '', '', '', '', '']
        widths = [1, 1, 1, 1, 1,
                  1, 1, 1, 1, 1]
        colors = ["blue", "red", "green", "black", "cyan",
                  "magenta", "yellow", "dark red", "dark green", "blue"]
        styles = [1, 1, 1, 1, 1,
                  1, 1, 1, 1, 1]
        markers = [-1, -1, -1, -1, -1,
                   -1, -1, -1, -1, -1]
        alphas = [1.0, 1.0, 1.0, 1.0, 1.0,
                  1.0, 1.0, 1.0, 1.0, 1.0]

        for i in xrange(1):
            if len(labels[i]) == 0:
                self.qtgui_time_sink_x_0.set_line_label(i, "Data {0}".format(i))
            else:
                self.qtgui_time_sink_x_0.set_line_label(i, labels[i])
            self.qtgui_time_sink_x_0.set_line_width(i, widths[i])
            self.qtgui_time_sink_x_0.set_line_color(i, colors[i])
            self.qtgui_time_sink_x_0.set_line_style(i, styles[i])
            self.qtgui_time_sink_x_0.set_line_marker(i, markers[i])
            self.qtgui_time_sink_x_0.set_line_alpha(i, alphas[i])

        self._qtgui_time_sink_x_0_win = sip.wrapinstance(self.qtgui_time_sink_x_0.pyqwidget(), Qt.QWidget)
        self.tab_grid_layout_1.addWidget(self._qtgui_time_sink_x_0_win, 0, 0, 1, 1)
        for r in range(0, 1):
            self.tab_grid_layout_1.setRowStretch(r, 1)
        for c in range(0, 1):
            self.tab_grid_layout_1.setColumnStretch(c, 1)
        self.low_pass_filter_2 = filter.fir_filter_ccf(4, firdes.low_pass(
        	1, rx_audio_sample_rate, rx_baud*1.5, rx_baud/2, firdes.WIN_HAMMING, 6.76))
        self.low_pass_filter_1 = filter.fir_filter_ccf(1, firdes.low_pass(
        	1, rx_audio_sample_rate, rx_baud/2 * 1.5, rx_baud * .2, firdes.WIN_HAMMING, 6.76))
        self.low_pass_filter_0_0 = filter.fir_filter_fff(lpf_dec, firdes.low_pass(
        	1, tx_audio_sample_rate, tx_baud*3, tx_baud/2, firdes.WIN_HAMMING, 6.76))
        self.low_pass_filter_0 = filter.fir_filter_fff(lpf_dec, firdes.low_pass(
        	1, rx_audio_sample_rate, rx_baud/2*1.5, rx_baud/2, firdes.WIN_HAMMING, 6.76))
        self.fft_filter_xxx_0_0 = filter.fft_filter_fff(1, (firdes.gaussian(1.0, rx_sps, 0.5, 4*rx_sps)), 1)
        self.fft_filter_xxx_0_0.declare_sample_delay(0)
        self.fft_filter_xxx_0 = filter.fft_filter_fff(1, (firdes.gaussian(1.0, tx_sps, 0.5, 4*tx_sps)), 1)
        self.fft_filter_xxx_0.declare_sample_delay(0)
        self.digital_symbol_sync_xx_0 = digital.symbol_sync_ff(digital.TED_MENGALI_AND_DANDREA_GMSK, rx_sps, 0.1, 0.5, 1.5, 0.3, 1, digital.constellation_bpsk().base(), digital.IR_MMSE_8TAP, 128, ([]))
        self.digital_chunks_to_symbols_xx_0 = digital.chunks_to_symbols_bf(([-1,1]), 1)
        self.digital_binary_slicer_fb_0 = digital.binary_slicer_fb()
        self.dc_blocker_xx_0 = filter.dc_blocker_ff(64000, False)
        self.blocks_unpack_k_bits_bb_0 = blocks.unpack_k_bits_bb(8)
        self.blocks_udp_source_0 = blocks.udp_source(gr.sizeof_char*1, '127.0.0.1', gr_transmit_port, 1472, False)
        self.blocks_udp_sink_0 = blocks.udp_sink(gr.sizeof_char*1, '127.0.0.1', gr_receive_port, 1472, True)
        self.blocks_repeat_0 = blocks.repeat(gr.sizeof_char*1, tx_sps)
        self.blocks_pack_k_bits_bb_0 = blocks.pack_k_bits_bb(8)
        self.blocks_null_source_0 = blocks.null_source(gr.sizeof_float*1)
        self.blocks_null_sink_0 = blocks.null_sink(gr.sizeof_float*1)
        self.blocks_multiply_xx_1 = blocks.multiply_vcc(1)
        self.blocks_multiply_xx_0 = blocks.multiply_vcc(1)
        self.blocks_multiply_const_vxx_1_0 = blocks.multiply_const_vff((0.75 if ssb_mode=="USB" else -0.75, ))
        self.blocks_multiply_const_vxx_1 = blocks.multiply_const_vff((0.75 if ssb_mode=="USB" else -0.75, ))
        self.blocks_float_to_complex_0 = blocks.float_to_complex(1)
        self.blocks_complex_to_float_0 = blocks.complex_to_float(1)
        self.audio_source_0_0 = audio.source(rx_audio_sample_rate, audiocard_rx_devicename, False)
        self.audio_sink_0 = audio.sink(tx_audio_sample_rate, audiocard_tx_devicename, True)
        self.analog_sig_source_x_1 = analog.sig_source_c(rx_audio_sample_rate, analog.GR_COS_WAVE, -ssb_rx_offset, 1, 0)
        self.analog_sig_source_x_0 = analog.sig_source_c(tx_audio_sample_rate, analog.GR_COS_WAVE, ssb_tx_offset, 1, 0)
        self.analog_quadrature_demod_cf_0 = analog.quadrature_demod_cf(1)
        self.analog_frequency_modulator_fc_0 = analog.frequency_modulator_fc(3.141592/tx_sps)
        self.analog_agc2_xx_0_0 = analog.agc2_ff(1e-1, 1e-2, 1.0, 1.0)
        self.analog_agc2_xx_0_0.set_max_gain(65536)



        ##################################################
        # Connections
        ##################################################
        self.connect((self.analog_agc2_xx_0_0, 0), (self.low_pass_filter_0, 0))
        self.connect((self.analog_frequency_modulator_fc_0, 0), (self.blocks_multiply_xx_0, 1))
        self.connect((self.analog_quadrature_demod_cf_0, 0), (self.blocks_multiply_const_vxx_1_0, 0))
        self.connect((self.analog_sig_source_x_0, 0), (self.blocks_multiply_xx_0, 0))
        self.connect((self.analog_sig_source_x_1, 0), (self.blocks_multiply_xx_1, 1))
        self.connect((self.audio_source_0_0, 0), (self.blocks_float_to_complex_0, 0))
        self.connect((self.blocks_complex_to_float_0, 0), (self.audio_sink_0, 0))
        self.connect((self.blocks_complex_to_float_0, 1), (self.blocks_null_sink_0, 0))
        self.connect((self.blocks_complex_to_float_0, 0), (self.qtgui_waterfall_sink_x_0_0, 0))
        self.connect((self.blocks_float_to_complex_0, 0), (self.blocks_multiply_xx_1, 0))
        self.connect((self.blocks_multiply_const_vxx_1, 0), (self.analog_frequency_modulator_fc_0, 0))
        self.connect((self.blocks_multiply_const_vxx_1_0, 0), (self.analog_agc2_xx_0_0, 0))
        self.connect((self.blocks_multiply_xx_0, 0), (self.blocks_complex_to_float_0, 0))
        self.connect((self.blocks_multiply_xx_1, 0), (self.low_pass_filter_1, 0))
        self.connect((self.blocks_multiply_xx_1, 0), (self.low_pass_filter_2, 0))
        self.connect((self.blocks_null_source_0, 0), (self.blocks_float_to_complex_0, 1))
        self.connect((self.blocks_pack_k_bits_bb_0, 0), (self.blocks_udp_sink_0, 0))
        self.connect((self.blocks_repeat_0, 0), (self.digital_chunks_to_symbols_xx_0, 0))
        self.connect((self.blocks_udp_source_0, 0), (self.blocks_unpack_k_bits_bb_0, 0))
        self.connect((self.blocks_unpack_k_bits_bb_0, 0), (self.blocks_repeat_0, 0))
        self.connect((self.dc_blocker_xx_0, 0), (self.fft_filter_xxx_0_0, 0))
        self.connect((self.digital_binary_slicer_fb_0, 0), (self.blocks_pack_k_bits_bb_0, 0))
        self.connect((self.digital_chunks_to_symbols_xx_0, 0), (self.fft_filter_xxx_0, 0))
        self.connect((self.digital_symbol_sync_xx_0, 0), (self.digital_binary_slicer_fb_0, 0))
        self.connect((self.fft_filter_xxx_0, 0), (self.low_pass_filter_0_0, 0))
        self.connect((self.fft_filter_xxx_0_0, 0), (self.digital_symbol_sync_xx_0, 0))
        self.connect((self.low_pass_filter_0, 0), (self.dc_blocker_xx_0, 0))
        self.connect((self.low_pass_filter_0, 0), (self.qtgui_time_sink_x_0_0, 0))
        self.connect((self.low_pass_filter_0_0, 0), (self.blocks_multiply_const_vxx_1, 0))
        self.connect((self.low_pass_filter_0_0, 0), (self.qtgui_time_sink_x_0, 0))
        self.connect((self.low_pass_filter_1, 0), (self.analog_quadrature_demod_cf_0, 0))
        self.connect((self.low_pass_filter_2, 0), (self.qtgui_waterfall_sink_x_0_0_0, 0))

    def closeEvent(self, event):
        self.settings = Qt.QSettings("GNU Radio", "top_block")
        self.settings.setValue("geometry", self.saveGeometry())
        event.accept()

    def get_configurationFilePath(self):
        return self.configurationFilePath

    def set_configurationFilePath(self, configurationFilePath):
        self.configurationFilePath = configurationFilePath
        self._tx_audio_sample_rate_config = ConfigParser.ConfigParser()
        self._tx_audio_sample_rate_config.read(self.configurationFilePath)
        if not self._tx_audio_sample_rate_config.has_section('gnuradio'):
        	self._tx_audio_sample_rate_config.add_section('gnuradio')
        self._tx_audio_sample_rate_config.set('gnuradio', 'AudioCard_TX_SampleRate', str(None))
        self._tx_audio_sample_rate_config.write(open(self.configurationFilePath, 'w'))
        self._ssb_tx_offset_config = ConfigParser.ConfigParser()
        self._ssb_tx_offset_config.read(self.configurationFilePath)
        if not self._ssb_tx_offset_config.has_section('gnuradio'):
        	self._ssb_tx_offset_config.add_section('gnuradio')
        self._ssb_tx_offset_config.set('gnuradio', 'SSB_TX_Offset', str(None))
        self._ssb_tx_offset_config.write(open(self.configurationFilePath, 'w'))
        self._ssb_rx_offset_config = ConfigParser.ConfigParser()
        self._ssb_rx_offset_config.read(self.configurationFilePath)
        if not self._ssb_rx_offset_config.has_section('gnuradio'):
        	self._ssb_rx_offset_config.add_section('gnuradio')
        self._ssb_rx_offset_config.set('gnuradio', 'SSB_RX_Offset', str(None))
        self._ssb_rx_offset_config.write(open(self.configurationFilePath, 'w'))
        self._ssb_mode_config = ConfigParser.ConfigParser()
        self._ssb_mode_config.read(self.configurationFilePath)
        if not self._ssb_mode_config.has_section('gnuradio'):
        	self._ssb_mode_config.add_section('gnuradio')
        self._ssb_mode_config.set('gnuradio', 'SSB_Mode', str(None))
        self._ssb_mode_config.write(open(self.configurationFilePath, 'w'))
        self._rx_audio_sample_rate_config = ConfigParser.ConfigParser()
        self._rx_audio_sample_rate_config.read(self.configurationFilePath)
        if not self._rx_audio_sample_rate_config.has_section('gnuradio'):
        	self._rx_audio_sample_rate_config.add_section('gnuradio')
        self._rx_audio_sample_rate_config.set('gnuradio', 'AudioCard_TX_SampleRate', str(None))
        self._rx_audio_sample_rate_config.write(open(self.configurationFilePath, 'w'))
        self._gr_transmit_port_config = ConfigParser.ConfigParser()
        self._gr_transmit_port_config.read(self.configurationFilePath)
        if not self._gr_transmit_port_config.has_section('modem'):
        	self._gr_transmit_port_config.add_section('modem')
        self._gr_transmit_port_config.set('modem', 'GR_TransmitPort', str(None))
        self._gr_transmit_port_config.write(open(self.configurationFilePath, 'w'))
        self._gr_receive_port_config = ConfigParser.ConfigParser()
        self._gr_receive_port_config.read(self.configurationFilePath)
        if not self._gr_receive_port_config.has_section('modem'):
        	self._gr_receive_port_config.add_section('modem')
        self._gr_receive_port_config.set('modem', 'GR_ReceivePort', str(None))
        self._gr_receive_port_config.write(open(self.configurationFilePath, 'w'))
        self._audiocard_tx_devicename_config = ConfigParser.ConfigParser()
        self._audiocard_tx_devicename_config.read(self.configurationFilePath)
        if not self._audiocard_tx_devicename_config.has_section('gnuradio'):
        	self._audiocard_tx_devicename_config.add_section('gnuradio')
        self._audiocard_tx_devicename_config.set('gnuradio', 'AudioCard_TX_DeviceName', str(None))
        self._audiocard_tx_devicename_config.write(open(self.configurationFilePath, 'w'))
        self._audiocard_rx_devicename_config = ConfigParser.ConfigParser()
        self._audiocard_rx_devicename_config.read(self.configurationFilePath)
        if not self._audiocard_rx_devicename_config.has_section('gnuradio'):
        	self._audiocard_rx_devicename_config.add_section('gnuradio')
        self._audiocard_rx_devicename_config.set('gnuradio', 'AudioCard_RX_DeviceName', str(None))
        self._audiocard_rx_devicename_config.write(open(self.configurationFilePath, 'w'))

    def get_rx_baud(self):
        return self.rx_baud

    def set_rx_baud(self, rx_baud):
        self.rx_baud = rx_baud
        self.set_rx_sps(self.rx_audio_sample_rate/self.rx_baud)
        self.set_sps(self.rx_audio_sample_rate/self.rx_baud)
        self.low_pass_filter_2.set_taps(firdes.low_pass(1, self.rx_audio_sample_rate, self.rx_baud*1.5, self.rx_baud/2, firdes.WIN_HAMMING, 6.76))
        self.low_pass_filter_1.set_taps(firdes.low_pass(1, self.rx_audio_sample_rate, self.rx_baud/2 * 1.5, self.rx_baud * .2, firdes.WIN_HAMMING, 6.76))
        self.low_pass_filter_0.set_taps(firdes.low_pass(1, self.rx_audio_sample_rate, self.rx_baud/2*1.5, self.rx_baud/2, firdes.WIN_HAMMING, 6.76))

    def get_rx_audio_sample_rate(self):
        return self.rx_audio_sample_rate

    def set_rx_audio_sample_rate(self, rx_audio_sample_rate):
        self.rx_audio_sample_rate = rx_audio_sample_rate
        self.set_rx_sps(self.rx_audio_sample_rate/self.rx_baud)
        self.set_sps(self.rx_audio_sample_rate/self.rx_baud)
        self.qtgui_waterfall_sink_x_0_0_0.set_frequency_range(0, self.rx_audio_sample_rate/4)
        self.qtgui_time_sink_x_0_0.set_samp_rate(self.rx_audio_sample_rate)
        self.low_pass_filter_2.set_taps(firdes.low_pass(1, self.rx_audio_sample_rate, self.rx_baud*1.5, self.rx_baud/2, firdes.WIN_HAMMING, 6.76))
        self.low_pass_filter_1.set_taps(firdes.low_pass(1, self.rx_audio_sample_rate, self.rx_baud/2 * 1.5, self.rx_baud * .2, firdes.WIN_HAMMING, 6.76))
        self.low_pass_filter_0.set_taps(firdes.low_pass(1, self.rx_audio_sample_rate, self.rx_baud/2*1.5, self.rx_baud/2, firdes.WIN_HAMMING, 6.76))
        self.analog_sig_source_x_1.set_sampling_freq(self.rx_audio_sample_rate)

    def get_tx_baud(self):
        return self.tx_baud

    def set_tx_baud(self, tx_baud):
        self.tx_baud = tx_baud
        self.set_tx_sps(self.tx_audio_sample_rate/self.tx_baud)
        self.low_pass_filter_0_0.set_taps(firdes.low_pass(1, self.tx_audio_sample_rate, self.tx_baud*3, self.tx_baud/2, firdes.WIN_HAMMING, 6.76))

    def get_tx_audio_sample_rate(self):
        return self.tx_audio_sample_rate

    def set_tx_audio_sample_rate(self, tx_audio_sample_rate):
        self.tx_audio_sample_rate = tx_audio_sample_rate
        self.set_tx_sps(self.tx_audio_sample_rate/self.tx_baud)
        self.qtgui_waterfall_sink_x_0_0.set_frequency_range(0, self.tx_audio_sample_rate)
        self.qtgui_time_sink_x_0.set_samp_rate(self.tx_audio_sample_rate)
        self.low_pass_filter_0_0.set_taps(firdes.low_pass(1, self.tx_audio_sample_rate, self.tx_baud*3, self.tx_baud/2, firdes.WIN_HAMMING, 6.76))
        self.analog_sig_source_x_0.set_sampling_freq(self.tx_audio_sample_rate)

    def get_sps(self):
        return self.sps

    def set_sps(self, sps):
        self.sps = sps
        self.set_rrc_filter(firdes.root_raised_cosine(self.nfilts, self.nfilts, 1.0/float(self.sps), 0.35, 11*self.nfilts))

    def get_nfilts(self):
        return self.nfilts

    def set_nfilts(self, nfilts):
        self.nfilts = nfilts
        self.set_rrc_filter(firdes.root_raised_cosine(self.nfilts, self.nfilts, 1.0/float(self.sps), 0.35, 11*self.nfilts))

    def get_variable_qtgui_range_0(self):
        return self.variable_qtgui_range_0

    def set_variable_qtgui_range_0(self, variable_qtgui_range_0):
        self.variable_qtgui_range_0 = variable_qtgui_range_0

    def get_tx_sps(self):
        return self.tx_sps

    def set_tx_sps(self, tx_sps):
        self.tx_sps = tx_sps
        self.fft_filter_xxx_0.set_taps((firdes.gaussian(1.0, self.tx_sps, 0.5, 4*self.tx_sps)))
        self.blocks_repeat_0.set_interpolation(self.tx_sps)
        self.analog_frequency_modulator_fc_0.set_sensitivity(3.141592/self.tx_sps)

    def get_ssb_tx_offset(self):
        return self.ssb_tx_offset

    def set_ssb_tx_offset(self, ssb_tx_offset):
        self.ssb_tx_offset = ssb_tx_offset
        self.analog_sig_source_x_0.set_frequency(self.ssb_tx_offset)

    def get_ssb_rx_offset(self):
        return self.ssb_rx_offset

    def set_ssb_rx_offset(self, ssb_rx_offset):
        self.ssb_rx_offset = ssb_rx_offset
        self.analog_sig_source_x_1.set_frequency(-self.ssb_rx_offset)

    def get_ssb_mode(self):
        return self.ssb_mode

    def set_ssb_mode(self, ssb_mode):
        self.ssb_mode = ssb_mode
        self.blocks_multiply_const_vxx_1_0.set_k((0.75 if self.ssb_mode=="USB" else -0.75, ))
        self.blocks_multiply_const_vxx_1.set_k((0.75 if self.ssb_mode=="USB" else -0.75, ))

    def get_rx_sps(self):
        return self.rx_sps

    def set_rx_sps(self, rx_sps):
        self.rx_sps = rx_sps
        self.fft_filter_xxx_0_0.set_taps((firdes.gaussian(1.0, self.rx_sps, 0.5, 4*self.rx_sps)))

    def get_rrc_filter(self):
        return self.rrc_filter

    def set_rrc_filter(self, rrc_filter):
        self.rrc_filter = rrc_filter

    def get_lpf_dec(self):
        return self.lpf_dec

    def set_lpf_dec(self, lpf_dec):
        self.lpf_dec = lpf_dec

    def get_gr_transmit_port(self):
        return self.gr_transmit_port

    def set_gr_transmit_port(self, gr_transmit_port):
        self.gr_transmit_port = gr_transmit_port

    def get_gr_receive_port(self):
        return self.gr_receive_port

    def set_gr_receive_port(self, gr_receive_port):
        self.gr_receive_port = gr_receive_port

    def get_bw_fact(self):
        return self.bw_fact

    def set_bw_fact(self, bw_fact):
        self.bw_fact = bw_fact

    def get_audiocard_tx_devicename(self):
        return self.audiocard_tx_devicename

    def set_audiocard_tx_devicename(self, audiocard_tx_devicename):
        self.audiocard_tx_devicename = audiocard_tx_devicename

    def get_audiocard_rx_devicename(self):
        return self.audiocard_rx_devicename

    def set_audiocard_rx_devicename(self, audiocard_rx_devicename):
        self.audiocard_rx_devicename = audiocard_rx_devicename


def main(top_block_cls=top_block, options=None):

    from distutils.version import StrictVersion
    if StrictVersion(Qt.qVersion()) >= StrictVersion("4.5.0"):
        style = gr.prefs().get_string('qtgui', 'style', 'raster')
        Qt.QApplication.setGraphicsSystem(style)
    qapp = Qt.QApplication(sys.argv)

    tb = top_block_cls()
    tb.start()
    tb.show()

    def quitting():
        tb.stop()
        tb.wait()
    qapp.connect(qapp, Qt.SIGNAL("aboutToQuit()"), quitting)
    qapp.exec_()


if __name__ == '__main__':
    main()
